/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  Spruce
 *  Copyright (C) 1999-2000 Jeffrey Stedfast
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* Coded to be compliant with RFC 1725 (Post Office Protocol - Version 3) */

#ifndef __POP3IO_H__
#define __POP3IO_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <fcntl.h>
#include <netdb.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
/*#include <sys/stat.h>*/
/*#include <sys/uio.h>*/
#include <unistd.h>
#include <sys/wait.h>
#include <sys/un.h>
/*#include <sys/param.h>*/
#undef MIN
#undef MAX
#include <glib.h>

#include "defines.h"
#include "parse.h"
#include "sockio.h"
#include "server.h"

#include "status.h"
#include "globals.h"
#include "mailbox.h"
#include "md5.h"
#include "accounts_t.h"
#include "cram-md5.h"

/* version information for pop3io */
#define P_VER_MAJOR '0'
#define P_VER_MINOR '5'
#define P_VER_REL   '0'

enum pop3_errnos {POP_NO_ERROR,
                  POP_ERR_RESOLVE,
                  POP_ERR_SOCKET,
                  POP_ERR_CONNECT,
                  POP_ERR_SERV_ACK, 
                  POP_ERR_USER_SEND,
                  POP_ERR_USER_RECV,
                  POP_ERR_PASS_SEND,
                  POP_ERR_PASS_RECV, 
                  POP_ERR_LIST_SEND,
                  POP_ERR_LIST_RECV,
                  POP_ERR_STAT_SEND,
                  POP_ERR_STAT_RECV,
                  POP_ERR_UIDL_SEND,
                  POP_ERR_UIDL_RECV,
                  POP_ERR_RETR_SEND,
                  POP_ERR_RETR_RECV,
                  POP_ERR_RSET_SEND,
                  POP_ERR_RSET_RECV, 
                  POP_ERR_QUIT_SEND,
                  POP_ERR_QUIT_RECV,
                  POP_ERR_DELE_SEND,
                  POP_ERR_DELE_RECV, 
                  POP_ERR_APOP_SEND, 
                  POP_ERR_APOP_RECV, 
                  POP_ERR_NO_APOP,
                  POP_ERR_AUTH_SEND,
                  POP_ERR_AUTH_RECV,
                  POP_ERR_NO_CRAM_MD5 };
					  
typedef enum pop3_errnos pop3errnos;

struct _emessage_
{
	gchar *header;
	gchar *body;
};

typedef struct _emessage_ Emesg;

/* grrr, we need these here because we cant include main.h */
void invoke_in_gui_thread (GtkFunction func, gpointer data);

gint mailthread_update_status_progress (gpointer data);

gint mailthread_update_status_text (gpointer data);

void pop3error (gchar *reason, gint len);

void emesg_destroy (Emesg *msg);

gint pop_auth (gint sock, gchar *login, gchar *password);

gint pop_apop (gint sock, gchar *login, gchar *password, gchar *challenge);

gint pop_user (gint sock, gchar *login);

gint pop_pass (gint sock, gchar *passwd);

gint pop_stat (gint sock);

gint *pop_stat_trim (gint sock, gint *num);

gint pop_list (gint sock);

gint pop_uidl (gint sock, guint id, gchar *uid);

gint pop_retr (gint sock, guint id, Mailbox_t *finbox, gboolean del_yn);

gint pop_dele (gint sock, guint id);

gint pop_rset (gint sock);

gint pop_quit (gint sock);

gint fetchmail (Server *server, gchar *login, gchar *passwd, Mailbox_t *finbox, 
		gboolean del_yn, gboolean show_dialog, conn_auth_t authtype);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __POP3IO_H__ */
